BROKER_URL = "redis://127.0.0.1:6379/3"

CELERYD_CONCURRENCY = 1
CELERYD_PREFETCH_MULTIPLIER = 1

CELERY_SEND_EVENTS = 1
CELERY_RESULT_BACKEND = BROKER_URL
CELERY_RESULT_SERIALIZER = 'json'
CELERY_TASK_SERIALIZER = 'json'
CELERY_ACCEPT_CONTENT = ['json', 'msgpack', 'yaml']
CELERY_IMPORTS = ('cw_celerytask_helpers.redislogger', 'demotasks')

CUBICWEB_CELERYTASK_LOG_ALL = True
CUBICWEB_CELERYTASK_REDIS_URL = BROKER_URL
